@@ -49,7 +49,8 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
                    const bool &any_event, const bool &any_interval,
                    const vec &W0H_bs_gammas, const vec &W0h_bs_gammas, const vec &W0H2_bs_gammas,
                    const vec &WH_gammas, const vec &Wh_gammas, const vec &WH2_gammas,
-                   const vec &log_Pwk, const vec &log_Pwk2,
+                   const vec &log_Pwk, const vec &log_Pwk2, const vec &log_weights,
+                   const uvec &id_h2, const uvec &intgr_ind, const bool &intgr,
                    const uvec &id_H_fast, const uvec &id_h_fast,
                    const uvec &which_event, const uvec &which_right_event, const uvec &which_left,
                    const uvec &which_interval, const field<uvec> &unq_idL,
@@ -142,7 +143,8 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
   logLik_surv = log_surv(W0H_bs_gammas, W0h_bs_gammas, W0H2_bs_gammas,
                          WH_gammas, Wh_gammas, WH2_gammas,
                          WlongH_alphas, Wlongh_alphas, WlongH2_alphas,
-                         log_Pwk, log_Pwk2, id_H_fast, id_h_fast,
+                         log_Pwk, log_Pwk2, log_weights, id_h2, intgr_ind, intgr,
+                         id_H_fast, id_h_fast,
                          which_event, which_right_event, which_left,
                          any_interval, which_interval,
                          recurrent, frailtyH_sigmaF_alphaF, frailtyh_sigmaF_alphaF);
@@ -208,7 +210,8 @@ void update_betas (field<vec> &betas, mat &res_betas, field<vec> &acceptance_bet
           log_surv(W0H_bs_gammas, W0h_bs_gammas, W0H2_bs_gammas,
                    WH_gammas, Wh_gammas, WH2_gammas,
                    WlongH_alphas_prop, Wlongh_alphas_prop, WlongH2_alphas_prop,
-                   log_Pwk, log_Pwk2, id_H_fast, id_h_fast,
+                   log_Pwk, log_Pwk2, log_weights, id_h2, intgr_ind, intgr,
+                   id_H_fast, id_h_fast,
                    which_event, which_right_event, which_left,
                    any_interval, which_interval,
                    recurrent, frailtyH_sigmaF_alphaF, frailtyh_sigmaF_alphaF);
