@@ -19,7 +19,7 @@ double logPrior_surv (
     const vec &prior_mean_alphas, mat &prior_Tau_alphas, const vec &lambda_alphas,
     const double &tau_alphas, const bool &shrink_alphas,
     const bool &recurrent, const vec &alphaF, const vec prior_mean_alphaF,
-    mat &prior_Tau_alphaF, const vec &lambda_alphaF, const double &tau_alphaF, 
+    mat &prior_Tau_alphaF, const vec &lambda_alphaF, const double &tau_alphaF,
     const bool &shrink_alphaF) {
   uword n_strata = prior_mean_bs_gammas.n_elem;
   uword n_per_stratum = bs_gammas.n_rows / n_strata;
@@ -46,7 +46,8 @@ void update_bs_gammas (vec &bs_gammas, const vec &gammas, const vec &alphas,
                        vec &W0H_bs_gammas, vec &W0h_bs_gammas, vec &W0H2_bs_gammas,
                        const vec &WH_gammas, const vec &Wh_gammas, const vec &WH2_gammas,
                        const vec &WlongH_alphas, const vec &Wlongh_alphas, const vec &WlongH2_alphas,
-                       const vec &log_Pwk, const vec &log_Pwk2,
+                       const vec &log_Pwk, const vec &log_Pwk2, const vec &log_weights,
+                       const uvec &id_h2, const uvec &intgr_ind, const bool &intgr,
                        const uvec &indFast_H, const uvec &indFast_h,
                        const uvec &which_event, const uvec &which_right_event,
                        const uvec &which_left, const uvec &which_interval,
@@ -82,7 +83,8 @@ void update_bs_gammas (vec &bs_gammas, const vec &gammas, const vec &alphas,
       log_surv(proposed_W0H_bs_gammas, proposed_W0h_bs_gammas, proposed_W0H2_bs_gammas,
                WH_gammas, Wh_gammas, WH2_gammas,
                WlongH_alphas, Wlongh_alphas, WlongH2_alphas,
-               log_Pwk, log_Pwk2, indFast_H, indFast_h,
+               log_Pwk, log_Pwk2, log_weights, id_h2, intgr_ind, intgr,
+               indFast_H, indFast_h,
                which_event, which_right_event, which_left,
                any_interval, which_interval,
                recurrent, frailtyH_sigmaF_alphaF, frailtyh_sigmaF_alphaF);
@@ -121,7 +123,8 @@ void update_gammas (const vec &bs_gammas, vec &gammas, const vec &alphas,
                     const vec &W0H_bs_gammas, const vec &W0h_bs_gammas, const vec &W0H2_bs_gammas,
                     vec &WH_gammas, vec &Wh_gammas, vec &WH2_gammas,
                     const vec &WlongH_alphas, const vec &Wlongh_alphas, const vec &WlongH2_alphas,
-                    const vec &log_Pwk, const vec &log_Pwk2,
+                    const vec &log_Pwk, const vec &log_Pwk2, const vec &log_weights,
+                    const uvec &id_h2, const uvec &intgr_ind, const bool &intgr,
                     const uvec &indFast_H, const uvec &indFast_h,
                     const uvec &which_event, const uvec &which_right_event,
                     const uvec &which_left, const uvec &which_interval,
@@ -156,7 +159,8 @@ void update_gammas (const vec &bs_gammas, vec &gammas, const vec &alphas,
       log_surv(W0H_bs_gammas, W0h_bs_gammas, W0H2_bs_gammas,
                proposed_WH_gammas, proposed_Wh_gammas, proposed_WH2_gammas,
                WlongH_alphas, Wlongh_alphas, WlongH2_alphas,
-               log_Pwk, log_Pwk2, indFast_H, indFast_h,
+               log_Pwk, log_Pwk2, log_weights, id_h2, intgr_ind, intgr,
+               indFast_H, indFast_h,
                which_event, which_right_event, which_left,
                any_interval, which_interval,
                recurrent, frailtyH_sigmaF_alphaF, frailtyh_sigmaF_alphaF);
@@ -196,7 +200,8 @@ void update_alphas (const vec &bs_gammas, const vec &gammas, vec &alphas,
                     const vec &W0H_bs_gammas, const vec &W0h_bs_gammas, const vec &W0H2_bs_gammas,
                     const vec &WH_gammas, const vec &Wh_gammas, const vec &WH2_gammas,
                     vec &WlongH_alphas, vec &Wlongh_alphas, vec &WlongH2_alphas,
-                    const vec &log_Pwk, const vec &log_Pwk2,
+                    const vec &log_Pwk, const vec &log_Pwk2, const vec &log_weights,
+                    const uvec &id_h2, const uvec &intgr_ind, const bool &intgr,
                     const uvec &indFast_H, const uvec &indFast_h,
                     const uvec &which_event, const uvec &which_right_event,
                     const uvec &which_left, const uvec &which_interval,
@@ -231,7 +236,8 @@ void update_alphas (const vec &bs_gammas, const vec &gammas, vec &alphas,
       log_surv(W0H_bs_gammas, W0h_bs_gammas, W0H2_bs_gammas,
                WH_gammas, Wh_gammas, WH2_gammas,
                proposed_WlongH_alphas, proposed_Wlongh_alphas, proposed_WlongH2_alphas,
-               log_Pwk, log_Pwk2, indFast_H, indFast_h,
+               log_Pwk, log_Pwk2, log_weights, id_h2, intgr_ind, intgr,
+               indFast_H, indFast_h,
                which_event, which_right_event, which_left,
                any_interval, which_interval,
                recurrent, frailtyH_sigmaF_alphaF, frailtyh_sigmaF_alphaF);
@@ -271,7 +277,8 @@ void update_alphaF (const vec &bs_gammas, const vec &gammas, const vec &alphas,
                     const vec &W0H_bs_gammas, const vec &W0h_bs_gammas, const vec &W0H2_bs_gammas,
                     const vec &WH_gammas, const vec &Wh_gammas, const vec &WH2_gammas,
                     const vec &WlongH_alphas, const vec &Wlongh_alphas, const vec &WlongH2_alphas,
-                    const vec &log_Pwk, const vec &log_Pwk2,
+                    const vec &log_Pwk, const vec &log_Pwk2, const vec &log_weights,
+                    const uvec &id_h2, const uvec &intgr_ind, const bool &intgr,
                     const uvec &indFast_H, const uvec &indFast_h,
                     const uvec &which_event, const uvec &which_right_event,
                     const uvec &which_left, const uvec &which_interval,
@@ -291,7 +298,7 @@ void update_alphaF (const vec &bs_gammas, const vec &gammas, const vec &alphas,
                     vec &alphaF, vec &alphaF_H, vec &alphaF_h,
                     vec &scale_alphaF, mat &acceptance_alphaF, mat &res_alphaF,
                     const vec prior_mean_alphaF, mat &prior_Tau_alphaF,
-                    const vec &lambda_alphaF, const double &tau_alphaF, 
+                    const vec &lambda_alphaF, const double &tau_alphaF,
                     const bool &shrink_alphaF,
                     const vec &sigmaF,
                     vec &frailtyH_sigmaF_alphaF, vec &frailtyh_sigmaF_alphaF) {
@@ -311,10 +318,11 @@ void update_alphaF (const vec &bs_gammas, const vec &gammas, const vec &alphas,
       log_surv(W0H_bs_gammas, W0h_bs_gammas, W0H2_bs_gammas,
                WH_gammas, Wh_gammas, WH2_gammas,
                WlongH_alphas, Wlongh_alphas, WlongH2_alphas,
-               log_Pwk, log_Pwk2, indFast_H, indFast_h,
+               log_Pwk, log_Pwk2, log_weights, id_h2, intgr_ind, intgr,
+               indFast_H, indFast_h,
                which_event, which_right_event, which_left,
                any_interval, which_interval,
-               recurrent, 
+               recurrent,
                proposed_frailtyH_sigmaF_alphaF, proposed_frailtyh_sigmaF_alphaF);
     double numerator_surv =
       sum(logLik_surv_proposed) +
@@ -339,7 +347,7 @@ void update_alphaF (const vec &bs_gammas, const vec &gammas, const vec &alphas,
       scale_alphaF.at(i) =
         robbins_monro(scale_alphaF.at(i),
                       acceptance_alphaF.at(it, i), it);
-      
+
     }
     // store results
     res_alphaF.at(it, i) = alphaF.at(i);
